@@ -8,9 +8,7 @@
 
 #include <access/heapam.h>
 #include <access/htup_details.h>
-#ifdef PGRN_SUPPORT_TABLEAM
-#	include <access/tableam.h>
-#endif
+#include <access/tableam.h>
 #include <catalog/pg_type.h>
 #include <executor/executor.h>
 #include <pgtime.h>
@@ -27,6 +25,46 @@
 #endif
 #include <utils/syscache.h>
 
+#include <string.h>
+
+void
+PGrnPGFullIndexNameSplit(const char *fullName,
+						 size_t fullNameSize,
+						 const char **indexName,
+						 size_t *indexNameSize,
+						 const char **attributeName,
+						 size_t *attributeNameSize)
+{
+	*indexName = NULL;
+	*indexNameSize = 0;
+	*attributeName = NULL;
+	*attributeNameSize = 0;
+
+	if (fullNameSize == 0)
+	{
+		return;
+	}
+
+	/* Should we use database encoding? */
+	{
+		const char *current = fullName;
+		const char *end = fullName + fullNameSize;
+		for (; current < end; current++)
+		{
+			if (current[0] == '.')
+				break;
+		}
+		*indexName = fullName;
+		*indexNameSize = current - fullName;
+		if (current == end) {
+			return;
+		}
+		/* +1/-1 is for '.' */
+		*attributeName = current + 1;
+		*attributeNameSize = end - current - 1;
+	}
+}
+
 Oid
 PGrnPGIndexNameToID(const char *name)
 {
@@ -247,7 +285,6 @@ PGrnPGDatumExtractString(Datum datum,
 bool
 PGrnPGHavePreparedTransaction(void)
 {
-#if PG_VERSION_NUM >= 120000
 	MemoryContext memoryContext;
 	MemoryContext oldMemoryContext;
 	FmgrInfo flinfo;
@@ -305,9 +342,6 @@ PGrnPGHavePreparedTransaction(void)
 	MemoryContextDelete(memoryContext);
 
 	return have;
-#else
-	return false;
-#endif
 }
 
 bool
@@ -315,3 +349,33 @@ PGrnPGIsParentIndex(Relation index)
 {
 	return PGRN_RELATION_GET_LOCATOR_NUMBER(index) == InvalidOid;
 }
+
+int
+PGrnPGResolveAttributeIndex(Relation index,
+							const char *name,
+							size_t nameSize)
+{
+	int i;
+
+	if (nameSize == 0)
+	{
+		return -1;
+	}
+
+	for (i = 0; i < index->rd_att->natts; i++)
+	{
+		const char *attributeName =
+			TupleDescAttr(index->rd_att, i)->attname.data;
+		if (strlen(attributeName) != nameSize)
+		{
+			continue;
+		}
+		if (memcmp(attributeName, name, nameSize) != 0)
+		{
+			continue;
+		}
+		return i;
+	}
+
+	return -1;
+}
